/*
 * Copyright notice from original mutt:
 * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
 * Copyright (C) 2004 g10 Code GmbH
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */
#ifndef _MUTT_CURSES_H
#define _MUTT_CURSES_H

#include "mutt.h"

/* AIX defines ``lines'' in <term.h>, but it's used as a var name in
 * various places in Mutt
 */
#ifdef lines
#undef lines
#endif /* lines */

#define CLEARLINE(W, x) wmove(W, x,0), wclrtoeol(W)
#define BEEP() do { if (mod_core.beep) beep(); } while (0)

#define BKGDSET(W, x)   wbkgdset(W, ColorDefs[x] | ' ')
#define SETCOLOR(W, X)  wattrset(W, ColorDefs[X])
#define ADDCOLOR(W, X)  wattron(W, ColorDefs[X])

void mutt_curs_set (int);

#define ctrl(c) ((c)-'@')

#ifdef KEY_ENTER
#define CI_is_return(c) ((c) == '\r' || (c) == '\n' || (c) == KEY_ENTER)
#else
#define CI_is_return(c) ((c) == '\r' || (c) == '\n')
#endif

event_t mutt_getch (void);
#ifndef waddnwstr
int waddwch(WINDOW *win, wchar_t w);
#else
#define waddwch(win, w)  waddnwstr(win, &(w), 1)
#endif

void curses_initialize(void);
void curses_install(void);

void mutt_flushinp (void);
void mutt_ungetch (int, int);

/* ----------------------------------------------------------------------------
 * Support for color
 */

enum {
  MT_COLOR_HDEFAULT = 0,
  MT_COLOR_QUOTED,
  MT_COLOR_SIGNATURE,
  MT_COLOR_INDICATOR,
  MT_COLOR_STATUS,
  MT_COLOR_TREE,
  MT_COLOR_NORMAL,
  MT_COLOR_ERROR,
  MT_COLOR_TILDE,
  MT_COLOR_MARKERS,
  MT_COLOR_BODY,
  MT_COLOR_HEADER,
  MT_COLOR_MESSAGE,
  MT_COLOR_ATTACHMENT,
  MT_COLOR_SEARCH,
  MT_COLOR_BOLD,
  MT_COLOR_SIDEBAR,
  MT_COLOR_UNDERLINE,
  MT_COLOR_INDEX,
  MT_COLOR_NEW,
  MT_COLOR_FLAGGED,
  MT_COLOR_MAX
};

typedef struct color_line {
  regex_t rx;
  char *pattern;
  pattern_t *color_pattern;     /* compiled pattern to speed up index color
                                   calculation */
  short fg;
  short bg;
  int pair;
  struct color_line *next;
} COLOR_LINE;

typedef struct {
  const char* msg;
  long pos;
  long size;
  char sizestr[STRING];
} progress_t;

void mutt_progress_bar (progress_t* progress, long pos);
void mutt_clear_error (void);
void mutt_edit_file (const char *);
void mutt_curses_error (const char *, ...)
    __attribute__((format(printf, 1, 2)));
void mutt_curses_message (const char *, ...)
    __attribute__((format(printf, 1, 2)));
void mutt_format_string (char *, ssize_t, int, int, int, char, const char *,
                         ssize_t, int);
void mutt_format_s (char *, ssize_t, const char *, const char *);
void mutt_format_s_tree (char *, ssize_t, const char *, const char *);
void mutt_free_color (int fg, int bg);
void mutt_paddstr(WINDOW *, int, const char *);
#define mutt_perror(a) _mutt_perror (a, __FILE__, __LINE__)
void _mutt_perror (const char*, const char*,int);
void mutt_query_exit (void);
void mutt_query_menu (char *, ssize_t);
void mutt_show_error (void);
ssize_t mutt_pretty_size(char *s, ssize_t len, ssize_t n);

int mutt_alloc_color (int fg, int bg);
int mutt_any_key_to_continue (const char *);
void mutt_what_key (void);
int mutt_complete (char *, ssize_t);
#define mutt_enter_fname(A,B,C,D,E) _mutt_enter_fname(A,B,C,D,E,0,NULL,NULL)
int _mutt_enter_fname (const char *, char *, ssize_t, int *, int, int,
                       char ***, int *);

#define mutt_get_field(A,B,C,D) _mutt_get_field(A,B,C,D,0,NULL,NULL)
int _mutt_get_field (const char *, char *, ssize_t, int, int, char ***, int *);
int mutt_get_field_unbuffered (char *, char *, ssize_t, int);
int mutt_index_menu (void);
int mutt_is_mail_list (address_t *);
int mutt_is_subscribed_list (address_t *);
int mutt_multi_choice (const char *prompt, const char *letters);
int mutt_parse_color (BUFFER *, BUFFER *, unsigned long, BUFFER *);
int mutt_parse_uncolor (BUFFER *, BUFFER *, unsigned long, BUFFER *);
int mutt_parse_mono (BUFFER *, BUFFER *, unsigned long, BUFFER *);
int mutt_parse_unmono (BUFFER *, BUFFER *, unsigned long, BUFFER *);
int mutt_query_complete (char *, ssize_t);
int mutt_user_is_recipient (HEADER *);
int mutt_yesorno (const char *, int);
void mutt_set_header_color (CONTEXT *, HEADER *);

extern int *ColorQuote;
extern int ColorQuoteUsed;
extern int ColorDefs[];
extern COLOR_LINE *ColorHdrList;
extern COLOR_LINE *ColorBodyList;
extern COLOR_LINE *ColorIndexList;

void ci_start_color (void);

#define mutt_make_string(A,B,C,D,E) _mutt_make_string(A,B,C,D,E,0)
void _mutt_make_string (char *, ssize_t, const char *, CONTEXT *,
                        HEADER *, format_flag);

#define MAYBE_REDRAW(x) if (option (OPTNEEDREDRAW)) { unset_option (OPTNEEDREDRAW); x = REDRAW_FULL; }

#endif /* !_MUTT_CURSES_H */
