/*
 * Copyright notice from original mutt:
 * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
 * Copyright (C) 2004 g10 Code GmbH
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

#ifndef _MUTT_H
#define _MUTT_H

#include <lib-lib/lib-lib.h>
#include <lib-mime/mime.h>
#include <lib-lua/lib-lua.h>

#define MUTT_VERSION (VERSION)

/* nifty trick I stole from ELM 2.5alpha. */
#ifdef MAIN_C
#define WHERE
#define INITVAL(x) = x
#else
#define WHERE extern
#define INITVAL(x)
#endif

typedef struct {
  int ch;                       /* raw key pressed */
  int op;                       /* function op */
} event_t;

/* types for mutt_add_hook() */
#define M_FOLDERHOOK	(1<<0)
#define M_MBOXHOOK	(1<<1)
#define M_SENDHOOK	(1<<2)
#define M_FCCHOOK	(1<<3)
#define M_SAVEHOOK	(1<<4)
#define M_MESSAGEHOOK	(1<<5)
#define M_CRYPTHOOK	(1<<6)
#define M_REPLYHOOK	(1<<7)
#define M_SEND2HOOK     (1<<8)

#define M_OPENHOOK	(1<<9)
#define M_APPENDHOOK	(1<<10)
#define M_CLOSEHOOK	(1<<11)

/* tree characters for linearize_tree and print_enriched_string */
#define M_TREE_LLCORNER		1
#define M_TREE_ULCORNER		2
#define M_TREE_LTEE		3
#define M_TREE_HLINE		4
#define M_TREE_VLINE		5
#define M_TREE_SPACE		6
#define M_TREE_RARROW		7
#define M_TREE_STAR		8
#define M_TREE_HIDDEN		9
#define M_TREE_EQUALS		10
#define M_TREE_TTEE		11
#define M_TREE_BTEE		12
#define M_TREE_MISSING		13
#define M_TREE_MAX		14

#define M_THREAD_COLLAPSE	(1<<0)
#define M_THREAD_UNCOLLAPSE	(1<<1)
#define M_THREAD_GET_HIDDEN	(1<<2)
#define M_THREAD_UNREAD		(1<<3)
#define M_THREAD_NEXT_UNREAD	(1<<4)

enum {
  /* modes for mutt_view_attachment() */
  M_REGULAR = 1,
  M_MAILCAP,
  M_AS_TEXT,

  /* action codes used by mutt_set_flag() and mutt_pattern_function() */
  M_ALL,
  M_NONE,
  M_NEW,
  M_OLD,
  M_REPLIED,
  M_READ,
  M_UNREAD,
  M_DELETE,
  M_UNDELETE,
  M_DELETED,
  M_APPENDED,
  M_PURGED,
  M_FLAG,
  M_TAG,
  M_UNTAG,
  M_LIMIT,
  M_EXPIRED,
  M_SUPERSEDED,

  /* actions for mutt_pattern_comp/mutt_pattern_exec */
  M_AND,
  M_OR,
  M_TO,
  M_CC,
  M_COLLAPSED,
  M_SUBJECT,
  M_FROM,
  M_DATE,
  M_DATE_RECEIVED,
  M_DUPLICATED,
  M_UNREFERENCED,
  M_ID,
  M_BODY,
  M_HEADER,
  M_HORMEL,
  M_WHOLE_MSG,
  M_SENDER,
  M_MESSAGE,
  M_SCORE,
  M_SIZE,
  M_REFERENCE,
  M_RECIPIENT,
  M_LIST,
  M_SUBSCRIBED_LIST,
  M_PERSONAL_RECIP,
  M_PERSONAL_FROM,
  M_ADDRESS,
  M_CRYPT_SIGN,
  M_CRYPT_VERIFIED,
  M_CRYPT_ENCRYPT,
  M_PGP_KEY,
  M_XLABEL,
  M_MIMEATTACH,
#ifdef USE_NNTP
  M_NEWSGROUPS,
#endif
  M_REALNAME,
  M_MULTIPART,

  /* Options for Mailcap lookup */
  M_EDIT,
  M_COMPOSE,
  M_PRINT,
  M_AUTOVIEW,

  /* options for socket code */
  M_NEW_SOCKET,

  /* Options for mutt_save_attachment */
  M_SAVE_APPEND,
  M_SAVE_OVERWRITE
};

/* quad-option vars */
enum {
  OPT_ABORT,
  OPT_ATTACH,
  OPT_BOUNCE,
  OPT_COPY,
  OPT_DELETE,
  OPT_FORWEDIT,
  OPT_INCLUDE,
  OPT_IMAPRECONNECT,
  OPT_MFUPTO,
  OPT_MIMEFWD,
  OPT_MIMEFWDREST,
  OPT_MOVE,
  OPT_POPDELETE,
  OPT_POPRECONNECT,
  OPT_POSTPONE,
  OPT_PRINT,
  OPT_REPLYTO,
  OPT_RECALL,
  OPT_SUBJECT,
  OPT_LISTREPLY,
#ifdef USE_NNTP
  OPT_TOMODERATED,
  OPT_NNTPRECONNECT,
  OPT_CATCHUP,
  OPT_FOLLOWUPTOPOSTER,
#endif                          /* USE_NNTP */

  /* THIS MUST BE THE LAST VALUE. */
  OPT_MAX
};

/* flags to ci_send_message() */
#define SENDREPLY	(1<<0)
#define SENDGROUPREPLY	(1<<1)
#define SENDLISTREPLY	(1<<2)
#define SENDFORWARD	(1<<3)
#define SENDPOSTPONED	(1<<4)
#define SENDBATCH	(1<<5)
#define SENDRESEND	(1<<6)
#define SENDNEWS	(1<<7)

/* flags to mutt_select_file() */
#define M_SEL_BUFFY	(1<<0)
#define M_SEL_MULTI	(1<<1)
#define M_SEL_FOLDER	(1<<2)

/* boolean vars */
enum {
  OPTALLOW8BIT,
  OPTALLOWANSI,
  OPTASKBCC,
  OPTASKCC,
  OPTASKFOLLOWUP,
  OPTATTACHSPLIT,
  OPTAUTOEDIT,
  OPTAUTOTAG,
  OPTBOUNCEDELIVERED,
  OPTCHECKNEW,
  OPTCOLLAPSEUNREAD,
  OPTCONFIRMAPPEND,
  OPTCONFIRMCREATE,
  OPTDELETEUNTAG,
  OPTDELSP,
  OPTDIGESTCOLLAPSE,
  OPTDUPTHREADS,
  OPTEDITHDRS,
  OPTENCODEFROM,
  OPTFASTREPLY,
  OPTFCCATTACH,
  OPTFCCCLEAR,
  OPTFOLLOWUPTO,
  OPTFORCEBUFFYCHECK,
  OPTFORCENAME,
  OPTFORWDECODE,
  OPTFORWQUOTE,
#ifdef HAVE_QDBM
  OPTHCACHECOMPRESS,
#endif
  OPTHEADER,
  OPTHIDDENHOST,
  OPTHIDELIMITED,
  OPTHIDEMISSING,
  OPTHIDETHREADSUBJECT,
  OPTHIDETOPLIMITED,
  OPTHIDETOPMISSING,
  OPTIGNORELISTREPLYTO,
  OPTIMAPCHECKSUBSCRIBED,
  OPTIMAPLSUB,
  OPTIMAPPASSIVE,
  OPTIMAPSERVERNOISE,
  OPTIMPLICITAUTOVIEW,
  OPTKEEPFLAGGED,
  OPTMAILDIRTRASH,
  OPTMARKERS,
  OPTMARKOLD,
  OPTMBOXPANE,
  OPTMENUSCROLL,                /* scroll menu instead of implicit next-page */
  OPTMENUMOVEOFF,
  OPTMETOO,
  OPTMHPURGE,
  OPTMIMEFORWDECODE,
  OPTNARROWTREE,
  OPTPAGERSTOP,
  OPTPIPEDECODE,
  OPTPIPESPLIT,
  OPTPOPLAST,
  OPTPRINTDECODE,
  OPTPRINTSPLIT,
  OPTPROMPTAFTER,
  OPTQUOTEEMPTY,
  OPTQUOTEQUOTED,
  OPTREADONLY,
  OPTREPLYSELF,
  OPTRESOLVE,
  OPTREVALIAS,
  OPTREVNAME,
  OPTREVREAL,
  OPTSIGDASHES,
  OPTSORTRE,
  OPTSPAMSEP,
  OPTSTATUSONTOP,
  OPTSTRICTMAILTO,
  OPTSTRICTTHREADS,
  OPTSTRIPWAS,
  OPTSTUFFQUOTED,
  OPTSUSPEND,
  OPTTEXTFLOWED,
  OPTTHOROUGHSRC,
  OPTTHREADRECEIVED,
  OPTTILDE,
  OPTUNCOLLAPSEJUMP,
  OPTUSEFROM,
#ifdef HAVE_LIBIDN
  OPTUSEIDN,
#endif
  OPTUSEIPV6,
  OPTWAITKEY,
  OPTWEED,
  OPTWRAP,
  OPTWRAPSEARCH,
  OPTWRITEBCC,                  /* write out a bcc header? */
  OPTXMAILER,
  OPTXMAILTO,                   /* 1 if $edit_header is forcebly set */
  OPTXTERMSETTITLES,

  /* PGP options */

  OPTPGPRETAINABLESIG,
  OPTFORWDECRYPT,
  OPTPGPSHOWUNUSABLE,

  /* news options */

#ifdef USE_NNTP
  OPTSHOWNEWNEWS,
  OPTSHOWONLYUNREAD,
  OPTLOADDESC,
#endif                          /* USE_NNTP */

  /* pseudo options */

  OPTAUXSORT,                   /* (pseudo) using auxillary sort function */
  OPTFORCEREFRESH,              /* (pseudo) refresh even during macros */
  OPTNOCURSES,                  /* (pseudo) when sending in batch mode */
  OPTNEEDREDRAW,                /* (pseudo) to notify caller of a submenu */
  OPTSEARCHREVERSE,             /* (pseudo) used by ci_search_command */
  OPTMSGERR,                    /* (pseudo) used by mutt_error/mutt_message */
  OPTSEARCHINVALID,             /* (pseudo) used to invalidate the search pat */
  OPTSIGNALSBLOCKED,            /* (pseudo) using by mutt_block_signals () */
  OPTSYSSIGNALSBLOCKED,         /* (pseudo) using by mutt_block_signals_system () */
  OPTNEEDRESORT,                /* (pseudo) used to force a re-sort */
  OPTRESORTINIT,                /* (pseudo) used to force the next resort to be from scratch */
  OPTVIEWATTACH,                /* (pseudo) signals that we are viewing attachments */
  OPTFORCEREDRAWINDEX,          /* (pseudo) used to force a redraw in the main index */
  OPTFORCEREDRAWPAGER,          /* (pseudo) used to force a redraw in the pager */
  OPTSORTSUBTHREADS,            /* (pseudo) used when $sort_aux changes */
  OPTNEEDRESCORE,               /* (pseudo) set when the `score' command is used */
  OPTATTACHMSG,                 /* (pseudo) used by attach-message */
  OPTHIDEREAD,                  /* (pseudo) whether or not hide read messages */
  OPTKEEPQUIET,                 /* (pseudo) shut up the message and refresh
                                 *          functions while we are executing an
                                 *          external program.
                                 */
  OPTMENUCALLER,                /* (pseudo) tell menu to give caller a take */
  OPTREDRAWTREE,                /* (pseudo) redraw the thread tree */
  OPTPGPCHECKTRUST,             /* (pseudo) used by pgp_select_key () */
  OPTUNBUFFEREDINPUT,           /* (pseudo) don't use key buffer */

#ifdef USE_NNTP
  OPTNEWS,                      /* (pseudo) used to change reader mode */
  OPTNEWSSEND,                  /* (pseudo) used to change behavior when posting */
#endif
  OPTSHORTENHIERARCHY,          /* set when to shorten "hierarchies" in the sidebar */
  OPTSIDEBARNEWMAILONLY,
  OPTMAX
};

#define mutt_bit_alloc(n) p_new(char, (n + 7) / 8)
#define mutt_bit_set(v,n) v[n/8] |= (1 << (n % 8))
#define mutt_bit_unset(v,n) v[n/8] &= ~(1 << (n % 8))
#define mutt_bit_toggle(v,n) v[n/8] ^= (1 << (n % 8))
#define mutt_bit_isset(v,n) (v[n/8] & (1 << (n % 8)))

#define set_option(x) mutt_bit_set(Options,x)
#define unset_option(x) mutt_bit_unset(Options,x)
#define toggle_option(x) mutt_bit_toggle(Options,x)
#define option(x) mutt_bit_isset(Options,x)

void mutt_init (int, string_list_t *);

typedef struct thread {
  unsigned int fake_thread:1;
  unsigned int duplicate_thread:1;
  unsigned int sort_children:1;
  unsigned int check_subject:1;
  unsigned int visible:1;
  unsigned int deep:1;
  unsigned int subtree_visible:2;
  unsigned int next_subtree_visible:1;
  struct thread *parent;
  struct thread *child;
  struct thread *next;
  struct thread *prev;
  HEADER *message;
  HEADER *sort_key;
} THREAD;

typedef struct compress_info compress_info;
typedef struct pattern_t pattern_t;

typedef struct {
  char *path;
  FILE *fp;
  time_t mtime;
  time_t mtime_cur;             /* used with maildir folders */
  off_t size;
  off_t vsize;
  char *pattern;                /* limit pattern string */
  pattern_t *limit_pattern;     /* compiled limit pattern */
  HEADER **hdrs;
  HEADER *last_tag;             /* last tagged msg. used to link threads */
  THREAD *tree;                 /* top of thread tree */
  hash_t *id_hash;                /* hash table by msg id */
  hash_t *subj_hash;              /* hash table by subject */
  hash_t *thread_hash;            /* hash table for threading */
  int *v2r;                     /* mapping from virtual to real msgno */
  int hdrmax;                   /* number of pointers in hdrs */
  int msgcount;                 /* number of messages in the mailbox */
  int vcount;                   /* the number of virtual messages */
  int tagged;                   /* how many messages are tagged? */
  int new;                      /* how many new messages? */
  int unread;                   /* how many unread messages? */
  int deleted;                  /* how many deleted messages */
  int appended;                 /* how many saved messages? */
  int flagged;                  /* how many flagged messages */
  int msgnotreadyet;            /* which msg "new" in pager, -1 if none */
  void *data;                   /* driver specific data */

  short magic;                  /* mailbox type */

  compress_info *cinfo;         /* compressed mbox module private data */
  char *realpath;               /* path to compressed mailbox */

  unsigned int locked:1;        /* is the mailbox locked? */
  unsigned int changed:1;       /* mailbox has been modified */
  unsigned int readonly:1;      /* don't allow changes to the mailbox */
  unsigned int dontwrite:1;     /* dont write the mailbox on close */
  unsigned int append:1;        /* mailbox is opened in append mode */
  unsigned int quiet:1;         /* inhibit status messages? */
  unsigned int collapsed:1;     /* are all threads collapsed? */
  unsigned int closing:1;       /* mailbox is being closed */
  unsigned int counting:1;      /* do we just want to cound? */
} CONTEXT;

/* for attachment counter */
typedef struct {
  char *major;
  int major_int;
  const char *minor;
  regex_t minor_rx;
} ATTACH_MATCH;

/* Flags for mutt_count_body_parts() */
#define M_PARTS_TOPLEVEL (1<<0) /* is the top-level part */
#define M_PARTS_RECOUNT (1<<1) /* force recount */

#include "protos.h"
#include "globals.h"

#endif /* !_MUTT_H */
